@@ -30,7 +30,7 @@ from itertools import *
 from functools import partial
 from calculate.lib.datavars import VariableError
 
-from calculate.lib.utils.mount import isMount
+from calculate.lib.utils.mount import isMount, Btrfs, BtrfsError
 from calculate.lib.utils.files import (removeDir,
                                        processProgress, STDOUT,
                                        typeFile, pathJoin, process,
@@ -424,7 +424,7 @@ class Distributive(object):
     def performFormat(self):
         pass
 
-    def formatPartition(self, dev, format="ext4", label="", boot=False):
+    def formatPartition(self, dev, format="ext4", label="", purpose=None):
         pass
 
     def rndString(self):
@@ -845,10 +845,11 @@ class FormatProcess(process):
     dos_id = "0"
     gpt_id = "0"
 
-    def __init__(self, dev, label=None, boot=False):
+    def __init__(self, dev, label=None, purpose=None, compression=None):
         self.dev = dev
         self._label = label
-        self.boot = boot
+        self.purpose = purpose
+        self.compression = compression
         super(FormatProcess, self).__init__(*self.format_command())
 
     def format_command(self):
@@ -856,10 +857,15 @@ class FormatProcess(process):
         if not cmd:
             raise DistributiveError(_("command '%s' not found") % cmd)
 
-        params = self.bootparam() if self.boot else self.param()
+        params = (self.bootparam()
+                  if self.purpose in ("root", "boot")
+                  else self.param())
         return list(traverse(
             [cmd] + [x for x in params if x]))
 
+    def postaction(self):
+        return True
+
     def bootparam(self):
         return self.param()
 
@@ -977,6 +983,18 @@ class FormatBtrfs(FormatProcessGeneric):
     def param(self):
         return "-f", self.get_label(), self.dev
 
+class FormatBtrfsCompressed(FormatBtrfs):
+
+    def postaction(self):
+        try:
+            if self.purpose != "boot" and self.compression:
+                Btrfs(self.dev).compression = self.compression
+            if self.purpose == "root":
+                # disable btrfs compression for /boot directory in partition
+                Btrfs(self.dev).set_compression("boot", "")
+        except BtrfsError as e:
+            raise DistributiveError(
+                _("Failed to set btrfs compression for {}").format(self.dev))
 
 class PartitionDistributive(Distributive):
     format_map = {
@@ -992,13 +1010,16 @@ class PartitionDistributive(Distributive):
         'ntfs-3g': FormatNtfs,
         'ntfs': FormatNtfs,
         'uefi': FormatUefi,
+        'btrfs': FormatBtrfs,
+        'btrfs-comp': FormatBtrfsCompressed,
         'swap': FormatSwap
     }
 
     def __init__(self, partition, parent=None, mdirectory=None,
                  check=False, multipartition=None, flagRemoveDir=True,
                  fileSystem="ext4", isFormat=True, systemId=None,
-                 rootLabel="Calculate", partitionTable=None):
+                 rootLabel="Calculate", partitionTable=None,
+                 compression=None):
         """Initialize partition distributive
 
         mdirectory - directory for mount
@@ -1015,6 +1036,7 @@ class PartitionDistributive(Distributive):
         self.DirectoryObject = None
         self.systemId = systemId
         self.rootLabel = rootLabel
+        self.compression = compression
         self.partitionTable = partitionTable
 
     def getType(self):
@@ -1108,8 +1130,10 @@ class PartitionDistributive(Distributive):
                 realMountPoint = None
                 if fileSystem != "swap":
                     realMountPoint = pathJoin(mdirectory, mountPoint)
-                    self._mountPartition(dev, realMountPoint,
-                                         "-t %s" % fileSystem)
+                    mapFS = {'btrfs-comp': 'btrfs'}
+                    self._mountPartition(
+                        dev, realMountPoint,
+                        "-t %s" % mapFS.get(fileSystem,fileSystem))
                 partObj = PartitionDistributive(dev, flagRemoveDir=False,
                                                 fileSystem=fileSystem,
                                                 isFormat=isFormat,
@@ -1142,6 +1166,8 @@ class PartitionDistributive(Distributive):
                                    dataPartitions))
         # if has separate /boot partition
         bootmp = ["/boot", "/"]
+        purpose_map = {"/boot": "boot",
+                       "/": "root"}
         # format all get partition
         for fileSystem, dev, newID, mp in formatPartitions:
             if fileSystem == "swap":
@@ -1152,14 +1178,14 @@ class PartitionDistributive(Distributive):
                 if dev == self.partition:
                     self.formatPartition(dev, format=fileSystem,
                                          label=self.rootLabel,
-                                         boot= mp in bootmp)
+                                         purpose=purpose_map.get(mp, None))
                 else:
                     if mp == '/var/calculate':
                         self.formatPartition(dev, format=fileSystem,
                                              label="Calculate")
                     else:
                         self.formatPartition(dev, format=fileSystem,
-                                             boot= mp in bootmp)
+                                             purpose=purpose_map.get(mp, None))
         # change system id for partitions
         changeidPartitions = map(lambda x: (x[NEWID], x[DEV], x[PARTTABLE]),
                                  filter(lambda x: x[NEWID],
@@ -1174,7 +1200,7 @@ class PartitionDistributive(Distributive):
             raise DistributiveError(
                 _("Failed to format %s: this partition is mounted") % dev)
 
-    def formatPartition(self, dev, format="ext4", label="", boot=False):
+    def formatPartition(self, dev, format="ext4", label="", purpose=None):
         """Format partition"""
         if not format in self.format_map:
             raise DistributiveError(
@@ -1189,11 +1215,13 @@ class PartitionDistributive(Distributive):
             raise DistributiveError(_("Failed to format the partition") +
                                     " %s:\n%s" % (dev, _("Permission denied")))
 
-        format_process = self.format_map[format](dev, label=label, boot=boot)
+        format_process = self.format_map[format](dev, label=label,
+            purpose=purpose, compression=self.compression)
         if format_process.failed():
             raise DistributiveError(
                 _("Failed to format the partition") +
                 " %s:\n%s" % (dev, format_process.readerr()))
+        format_process.postaction()
 
     def performFormat(self):
         """Perform format for all partition of this distributive"""
@@ -1201,7 +1229,7 @@ class PartitionDistributive(Distributive):
             self.formatAllPartitions()
         elif self.isFormat:
             self.formatPartition(self.partition, format=self.fileSystem,
-                                 label=self.rootLabel, boot=True)
+                                 label=self.rootLabel, purpose="root")
             if self.systemId:
                 self.changeSystemID(self.partition, self.systemId,
                                     self.partitionTable)
