@@ -67,7 +67,17 @@ class FileSystemManager(object):
                   'label': '-L {labelname}',
                   'msdos': '83',
                   'ssd': ['ssd', 'discard', 'space_cache'],
-                  'type': ['hdd', 'usb-hdd']},
+                  'type': ['hdd', 'usb-hdd'],
+                  'compatible': ['btrfs-comp']},
+        'btrfs-comp': {'defaultopt': defaultOpt,
+                  'format': '/sbin/mkfs.btrfs',
+                  'formatparam': '{labelparam} -f {device}',
+                  'gpt': '8300',
+                  'label': '-L {labelname}',
+                  'msdos': '83',
+                  'ssd': ['ssd', 'discard', 'space_cache'],
+                  'type': ['hdd', 'usb-hdd'],
+                  'compatible': ['btrfs']},
         'jfs': {'defaultopt': defaultOpt,
                 'format': '/sbin/mkfs.jfs',
                 'formatparam': '{labelparam} -f {device}',
