@@ -34,7 +34,8 @@ from calculate.lib.utils.device import (getPartitionSize,
                                         getUUIDDict)
 from calculate.install.variables.autopartition import Sizes
 from calculate.lib.utils.files import getProgPath
-from calculate.lib.utils.mount import isMount, FStab, DiskSpace
+from calculate.lib.utils.mount import isMount, FStab, DiskSpace, Btrfs, \
+    BtrfsError
 from calculate.install.fs_manager import FileSystemManager
 
 from calculate.lib.cl_lang import setLocalTranslate, _
@@ -577,9 +578,16 @@ class VariableOsDiskFormat(ReadonlyVariable):
 
         def getFormat(dev):
             prop = device.udev.get_device_info(name=dev)
-            return prop.get('FSTAB_TYPE') or \
-                   fstab.getBy(what=fstab.TYPE, eq=dev) or \
-                   prop.get('ID_FS_TYPE', '')
+            fs = prop.get('FSTAB_TYPE') or \
+                 fstab.getBy(what=fstab.TYPE, eq=dev) or \
+                 prop.get('ID_FS_TYPE', '')
+            if fs == "btrfs":
+                try:
+                    if Btrfs(dev).compression != "":   
+                        return "btrfs-comp"
+                except BtrfsError:
+                    pass
+            return fs
 
         return map(getFormat,
                    self.Get('os_disk_dev'))
@@ -2548,3 +2556,13 @@ class VariableOsInstallFormatSingleSet(Variable):
                                           "USB Flash"))
                 except DistributiveError:
                     pass
+
+class VariableOsInstallBtrfsCompression(Variable):
+    """
+    Алгоритм сжатия для btrfs, в которых выбрано использовать сжатие
+    """
+    type = "choice"
+    value = "zstd"
+
+    def choice(self):
+        return ["zlib","lzo","zstd"]
